import { z } from 'zod';

export const packSchema = z.object({
	index: z.object({
		file: z.string(),
		'hash-format': z.enum(['sha256', 'sha512', 'sha1', 'md5', 'murmur2']),
		hash: z.string().regex(/^[a-fA-F0-9]+$/)
	}),
	name: z.string(),
	'pack-format': z.string().default('packwiz:1.0.0'),
	versions: z.object({
		minecraft: z.string(),
		fabric: z.string().optional(),
		forge: z.string().optional(),
		liteloader: z.string().optional(),
		quilt: z.string().optional()
	}),
	author: z.string().optional(),
	description: z.string().optional(),
	version: z.string().optional()
});

export const indexSchema = z.object({
	'hash-format': z.enum(['sha256', 'sha512', 'sha1', 'md5', 'murmur2']),
	files: z
		.object({
			file: z.string(),
			hash: z.string().regex(/^[a-fA-F0-9]+$/),
			alias: z.string(),
			'hash-format': z.enum(['sha256', 'sha512', 'sha1', 'md5', 'murmur2']),
			metafile: z.boolean().default(false),
			preserve: z.boolean().default(false)
		})
		.array(),
	update: z
		.object({
			curseforge: z
				.object({
					'file-id': z.number(),
					'project-id': z.number()
				})
				.optional(),
			modrinth: z
				.object({
					'mod-id': z.string(),
					version: z.string()
				})
				.optional()
		})
		.default({})
});

// export const metafileSchema = z.object({
//     download: z.object({
//         'hash-format': z.enum(['sha256', 'sha512', 'sha1', 'md5', 'murmur2']),
//         hash: z.string().regex(/^[a-fA-F0-9]+$/),
//         url: z.string().url().optional(),
//         mode: z.enum(['url', 'metadata:curseforge']).default('url')
//     }),
// 	filename: z.string(),
// 	name: z.string(),
// 	option: z
// 		.object({
// 			optional: z.boolean(),
// 			default: z.boolean().default(false),
// 			description: z.string().optional()
// 		})
// 		.default({ optional: false }),
// 	side: z.enum(['server', 'client', 'both']).default('both'),
//     update: z
//         .object({
//             curseforge: z
//                 .object({
//                     'file-id': z.bigint(),
//                     'project-id': z.bigint()
//                 })
//                 .optional(),
//             modrinth: z
//                 .object({
//                     'mod-id': z.string(),
//                     version: z.string()
//                 })
//                 .optional()
//         })
//         .default({})
// });

const metafileBase = z.object({
	filename: z.string(),
	name: z.string(),
	option: z
		.object({
			optional: z.boolean(),
			default: z.boolean().default(false),
			description: z.string().optional()
		})
		.default({ optional: false }),
	side: z.enum(['server', 'client', 'both']).default('both')
});

export const metafileSchema = z.union([
	metafileBase.extend({
		download: z.object({
			'hash-format': z.enum(['sha256', 'sha512', 'sha1', 'md5', 'murmur2']),
			hash: z.string().regex(/^[a-fA-F0-9]+$/),
			url: z.string().url(),
			mode: z.literal('url').default('url')
		}),
		update: z
			.object({
				curseforge: z
					.object({
						'file-id': z.bigint(),
						'project-id': z.bigint()
					})
					.optional(),
				modrinth: z
					.object({
						'mod-id': z.string(),
						version: z.string()
					})
					.optional()
			})
			.default({})
	}),
	metafileBase.extend({
		download: z.object({
			'hash-format': z.enum(['sha256', 'sha512', 'sha1', 'md5', 'murmur2']),
			hash: z.string().regex(/^[a-fA-F0-9]+$/),
			mode: z.literal('metadata:curseforge')
		}),
		update: z.object({
			curseforge: z.object({
				'file-id': z.bigint(),
				'project-id': z.bigint()
			}),
			modrinth: z
				.object({
					'mod-id': z.string(),
					version: z.string()
				})
				.optional()
		})
	})
]);
